	list p=12f615
	radix dec
#include "p12f615.inc"

	__config _INTRC_OSC_NOCLKOUT & _WDT_ON & _PWRTE_OFF & _MCLRE_OFF & _CP_OFF & _IOSCFS4 & _BOR_OFF

	org 0x0000       ;Reset vector
	goto main

	org 0x0004       ;Interrupt vector
	retfie

;=============================================================
; Constants
;=============================================================
IR_LED_PIN  equ 2
RF_DATA_PIN equ 4
RF_WAKE_PIN equ 5

;=============================================================
; Global variables
;=============================================================
counter  equ 0x7D     ;24-bit counter for delay loops
                      ; Also 8-bit counter for RF signal acquisition

rf_state equ 0x7C     ;Prior state flag for RF edge detection

	org 0x0005

main:

;=============================================================
; I. Initialize 
;     - Initialize watchdog timer prescaler
;     - Initialize I/O pins:
;         GP2: Output, low (Debug)
;         //GP2: Input (but only until PWM is activated)
;         GP4: Input
;         GP5: Output, low
;     - Initialize Timer1 to (1:8 prescaler)
;=============================================================
	clrf  GPIO

	bsf   STATUS, RP0    ; Select bank 1
    
    movlw 0xF9           ; 11111001 : WDT prescaler = 1:2 (36ms)
	movwf OPTION_REG

    clrf  ANSEL          ; Set all pins to digital I/O
    movlw 0x1B
    movwf TRISIO         ; Set GP2(LED) and GP5(RF wake) to output mode

	bcf   STATUS, RP0    ; Select bank 0

    movlw 0x30           ; 00110000 : Timer1 = 1:8 prescaler, internal clock, no gate, stopped
    movwf T1CON

;=============================================================
; II. Acquire signal
;  Note: Wake-to-sleep operation is only 14ms, so no need to
;        worry about the watchdog timer.
;     - Wake up
;       * Bring the receiver out of shutdown
;       * Wait 3ms for receiver to wake up
;       * Initialize Timer2 to 11ms and start
;       * Clear signal acquisition counters
;     - Measure signal
;      .* Measure peak-to-peak or until Timer2 times out.
;      .* Clear counters when peak-to-peak is not within 10%
;      .   of 0.9 to 1.1ms
;      .* If 8 or more consecutive peak-to-peak by the end,
;      .   jump to signal sync phase
;     - Go back to sleep for 3x36ms (108ms)
;      .* Put the receiver in shutdown
;      .* Disable Timer2
;=============================================================
acquire_signal:
	bsf   GPIO, RF_WAKE_PIN	;Wake up the receiever

    bsf   STATUS, RP0
	movlw 250
	movwf PR2               ; Timer 2 period = 250 (1ms after 1:4 prescaler)
	bcf   STATUS, RP0

    bcf   PIR1, TMR2IF      ; Clear Timer2 interrupt flag
	clrf  TMR2              ; Clear Timer2 counter
	movlw 0x15              ; 00010101 : Timer2 = 1:4 prescaler, 1:3 postscaler (3ms), running
	movwf T2CON

rf_startup_delay:
	btfss PIR1, TMR2IF
	goto rf_startup_delay   ; Wait 3ms for RF receiver to wake up

	movlw 0x55              ; 01010101 : Timer2 = 1:4 prescaler, 1:11 postscaler (11ms), running
	movwf T2CON
	bcf   PIR1, TMR2IF      ; Clear Timer2 interrupt flag

    clrf  counter           ; Clear the acquisition counter
    clrf  rf_state          ; Also clear the state variable

measure_preamble:
    clrf TMR1L
    clrf TMR1H				; Clear Timer1
	bsf  T1CON, TMR1ON		; Start Timer1

measure_preamble_wait:
	btfsc PIR1, TMR2IF                  ; Check for timeout
	goto  measure_preamble_timeout
	btfsc rf_state, 0
	goto  measure_preamble_wait_norise  ; Skip rise handling if prior state is high (can't have rising edge)
    btfss GPIO, RF_DATA_PIN
	goto  measure_preamble_wait_norise  ; Skip rise handling if RF data pin is low (not rising edge)

    bcf   T1CON, TMR1ON      ; Stop Timer1

   ;Rise handler
    clrw
    xorwf TMR1H, W
	btfss STATUS, Z
    goto  measure_preamble_reset ; Reset if longer than 255 cycles (>2.04ms)

    movlw 116
    subwf TMR1L, W               ; W = TMR1L - 119(=952/8)ms, carry clear if underflow (bad)
    btfss STATUS, C
    goto measure_preamble_reset

    movlw 134
    subwf TMR1L, W               ; W = TMR1L - 131(=1048/8)ms, carry clear if underflow(good)
    btfsc STATUS, C
	goto measure_preamble_reset

    incf  counter, F             ; We found a good pulse - increment the counter
	bsf   rf_state,0             ; Set state to high (we just found a rising edge)
	goto  measure_preamble

measure_preamble_reset:
    clrf  counter
	bsf   rf_state,0
	goto  measure_preamble

measure_preamble_wait_norise:
    btfss GPIO, RF_DATA_PIN
	clrf  rf_state                    ; Update the state variable
    goto  measure_preamble_wait

measure_preamble_timeout:
	movlw 3
	subwf counter, W        ; Sets carry if counter is at least 8
    btfsc STATUS, C
    goto  sync_signal
    ;call debug_blink

shutdown: ;-                ; No signal acquired - go to sleep for 108ms and try again
	clrf  GPIO              ; Put the receiver to sleep and turn the LED off
	bcf   T2CON, TMR2ON     ; Disable Timer2

    ;sleep
	;sleep
	sleep                   ; Sleep for 108ms
	goto acquire_signal

;=============================================================
; III. Synchronize to signal
;     - 
;=============================================================
sync_signal:
    call debug_blink
    call debug_blink
    goto shutdown

;=============================================================
; IV. Read 32 Manchester-encoded bits
;     - 
;=============================================================


debug_blink:
    bsf   GPIO, IR_LED_PIN
    call delay_100ms
    bcf   GPIO, IR_LED_PIN
    call delay_100ms
    return

delay_100ms:
    movlw 0xDD
	movwf 0x40
	movlw 0x82
	movwf 0x41
delay_100ms_loop:
	decfsz 0x40, F
	goto delay_100ms_loop
    clrwdt
	decfsz 0x41, F
	goto delay_100ms_loop
	return
	end
